function [x_lim, LQR_control] = hysteresis(x,limit,LQR_state)
% hysteresis is a function designed for Energy Based Controller Lai, Wu,
% She & Wu, which gives the desired state limitation for use in LQR
% control. The function defines the limits of the LQR. As the system 
% reaches the region of attraction (ROA) of the LQR, the function moves the   
% system limits to enlarge the controller workspace when LQR stabilization  
% is activated. The hystereses changes the state limits to max when the LQR  
% is activated.
%
% x     - state vector
% limit - vector of state limits, either ROA limits or maximum limits
% LQR_state - 'true' if the controller is in LQR mode, 'false' otherwise 
%
% x_lim is the output vector where state limits are positively defined:
%   
%   x_lim = [x1_limit x2_limit ... xn_limit]
%   control_state = LQR_state
%
% Written by: Jrgen Herje Nilsen
% Date: 19.03.2014
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


x1_max_lim = limit(1);
x2_max_lim = limit(2);
x1_LQR_lim = limit(3);
x2_LQR_lim = limit(4);

if mod(abs(x(1)),2*pi) <= x1_LQR_lim && mod(abs(x(2)),2*pi) <= x2_LQR_lim
    x_lim = [x1_max_lim x2_max_lim];
    LQR_control = 1;    
elseif mod(abs(x(1)),2*pi)<=x1_max_lim && mod(abs(x(2)),2*pi)<=x2_max_lim && LQR_state==true
    x_lim = [x1_max_lim x2_max_lim];
    LQR_control = 1;
elseif mod(abs(x(1)),2*pi) > x1_max_lim || mod(abs(x(2)),2*pi) > x2_max_lim
    x_lim = [x1_LQR_lim x2_LQR_lim];
    LQR_control = 0;
else
    x_lim = [x1_LQR_lim x2_LQR_lim];
    LQR_control = 0;
end

%----- COMMENTS -----:
% Bruker mod(abs(x1),2*pi) fordi det skal vre modulus av den absolutte
% verdien da man sjekker at den absolutte verdien skal vre innen 0 til
% 2*pi og ikke abs(mod(x1,2*pi)) som vil finne positiv verdi av modulus,
% som alltid vil vre innen 0 til 2 pi. Negativ verdi p x1 vil gi den
% tilsvarende verdien i omrdet 0 til 2*pi ved bruk av mod(x1,2*pi), mens
% mod(abs(x1),2*pi) vil kun resultere i korrekt fring av grenser nr
% abs(x1) er strre enn 2*pi, for  gi tilsvarende vinkel innenfor omrdet
% 0 til 2*pi.
